@@ -4,7 +4,7 @@
  * Copyright (c) 2004-2008 Mark Kretschmann <kretschmann@kde.org>                       *
  * Copyright (c) 2006,2008 Ian Monroe <ian@monroe.nu>                                   *
  * Copyright (c) 2008 Jason A. Donenfeld <Jason@zx2c4.com>                              *
- * Copyright (c) 2009 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                    *
+ * Copyright (c) 2009 Nikolaj Hald Nielsen <nhn@kde.org>                                *
  * Copyright (c) 2009 Artur Szymiec <artur.szymiec@gmail.com>                           *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
@@ -27,6 +27,7 @@
 #include "Amarok.h"
 #include "amarokconfig.h"
 #include "collection/CollectionManager.h"
+#include "Components.h"
 #include "statusbar/StatusBar.h"
 #include "Debug.h"
 #include "MainWindow.h"
@@ -57,25 +58,23 @@ namespace The {
     EngineController* engineController() { return EngineController::instance(); }
 }
 
-EngineController* EngineController::s_instance = 0;
-
 EngineController*
 EngineController::instance()
 {
-    return s_instance ? s_instance : new EngineController();
+    return Amarok::Components::engineController();
 }
 
 void
 EngineController::destroy()
 {
-    delete s_instance;
-    s_instance = 0;
+    //nothing to do?
 }
 
 EngineController::EngineController()
     : m_playWhenFetched( true )
     , m_fadeoutTimer( new QTimer( this ) )
     , m_volume( 0 )
+    , m_currentIsAudioCd( false )
 {
     DEBUG_BLOCK
 
@@ -84,8 +83,6 @@ EngineController::EngineController()
     m_fadeoutTimer->setSingleShot( true );
 
     connect( m_fadeoutTimer, SIGNAL( timeout() ), SLOT( slotStopFadeout() ) );
-
-    s_instance = this;
 }
 
 EngineController::~EngineController()
@@ -132,7 +129,7 @@ EngineController::initializePhonon()
 
     //Add an equalizer effect if available
     QList<Phonon::EffectDescription> mEffectDescriptions = Phonon::BackendCapabilities::availableAudioEffects();
-    foreach ( Phonon::EffectDescription mDescr, mEffectDescriptions ) {
+    foreach ( const Phonon::EffectDescription &mDescr, mEffectDescriptions ) {
         if ( mDescr.name() == QLatin1String( "KEqualizer" ) ) {
             m_equalizer = new Phonon::Effect( mDescr );
             eqUpdate();
@@ -210,25 +207,46 @@ EngineController::canDecode( const KUrl &url ) //static
     if( !item.isLocalFile() )
         return true;
 
+    // Filter the available mime types to only include audio and video, as amarok does not intend to play photos
+    static QStringList mimeTable = supportedMimeTypes();
+
+    const KMimeType::Ptr mimeType = item.mimeTypePtr();
+    
+    bool valid = false;
+    foreach( const QString &type, mimeTable )
+    {
+        if( mimeType->is( type ) )
+        {
+            valid = true;
+            break;
+        }
+    }
+
+    return valid;
+}
+
+QStringList
+EngineController::supportedMimeTypes()
+{
+    //NOTE this function must be thread-safe
     // Filter the available mime types to only include audio and video, as amarok does not intend to play photos
     static QStringList mimeTable = Phonon::BackendCapabilities::availableMimeTypes().filter( "audio/", Qt::CaseInsensitive ) +
                                    Phonon::BackendCapabilities::availableMimeTypes().filter( "video/", Qt::CaseInsensitive );
 
     // Add whitelist hacks
     mimeTable << "audio/x-m4b"; // MP4 Audio Books have a different extension that KFileItem/Phonon don't grok
-    //mimeTable << "?/?"; //Add comment
 
-    const QString mimeType = item.mimetype();
-    const bool valid = mimeTable.contains( mimeType, Qt::CaseInsensitive );
-
-    //we special case this as otherwise users hate us
-    if ( !valid && ( mimeType == "audio/mp3" || mimeType == "audio/x-mp3" ) && !installDistroCodec() )
+    // We special case this, as otherwise the users would hate us
+    if( ( !mimeTable.contains( "audio/mp3" ) || !mimeTable.contains( "audio/x-mp3" ) ) && !installDistroCodec() )
+    {
         The::statusBar()->longMessage(
                 i18n( "<p>Phonon claims it <b>cannot</b> play MP3 files. You may want to examine "
                       "the installation of the backend that phonon uses.</p>"
                       "<p>You may find useful information in the <i>FAQ</i> section of the <i>Amarok Handbook</i>.</p>" ), StatusBar::Error );
+        mimeTable << "audio/mp3" << "audio/x-mp3";
+    }
 
-    return valid;
+    return mimeTable;
 }
 
 bool
@@ -325,6 +343,7 @@ EngineController::play( const Meta::TrackPtr& track, uint offset )
         return;
 
     m_currentTrack = track;
+    m_currentIsAudioCd = false;
     delete m_boundedPlayback;
     delete m_multiPlayback;
     delete m_multiSource;
@@ -373,9 +392,12 @@ EngineController::playUrl( const KUrl &url, uint offset )
     resetFadeout();
 
     debug() << "URL: " << url.url();
+    debug() << "offset: " << offset;
 
     if ( url.url().startsWith( "audiocd:/" ) )
     {
+
+        m_currentIsAudioCd = true;
         //disconnect this signal for now or it will cause a loop that will cause a mutex lockup
         disconnect( m_controller, SIGNAL( titleChanged( int ) ), this, SLOT( slotTitleChanged( int ) ) );
 
@@ -403,8 +425,11 @@ EngineController::playUrl( const KUrl &url, uint offset )
         int trackNumber = parts.at( 1 ).toInt();
 
         debug() << "3.2.1...";
-        m_media->clear();
-        m_media->setCurrentSource( Phonon::Cd );
+        if( m_media->currentSource().type() != Phonon::MediaSource::Disc )
+        {
+            m_media->clear();
+            m_media->setCurrentSource( Phonon::Cd );
+        }
         debug() << "boom?";
         m_controller->setCurrentTitle( trackNumber );
         debug() << "no boom?";
@@ -431,10 +456,15 @@ EngineController::playUrl( const KUrl &url, uint offset )
 
     if( offset )
     {
+        debug() << "seeking to " << offset;
         m_media->pause();
         m_media->seek( offset );
     }
     m_media->play();
+
+    debug() << "track pos after play: " << trackPositionMs();
+
+
 }
 
 void
@@ -448,6 +478,7 @@ EngineController::stop( bool forceInstant ) //SLOT
 {
     DEBUG_BLOCK
 
+    m_currentIsAudioCd = false;
     // need to get a new instance of multi if played again
     delete m_multiPlayback;
     delete m_multiSource;
@@ -488,7 +519,10 @@ EngineController::stop( bool forceInstant ) //SLOT
         stateChangedNotify( Phonon::StoppedState, m_media->state() ); //immediately disable Stop action
     }
     else
+    {
         m_media->stop();
+        m_media->setCurrentSource( Phonon::MediaSource() );
+    }
 
     m_currentTrack = 0;
 }
@@ -532,7 +566,13 @@ EngineController::seek( int ms ) //SLOT
         int seekTo;
 
         if ( m_boundedPlayback )
+        {
             seekTo = m_boundedPlayback->startPosition() + ms;
+            if( seekTo < m_boundedPlayback->startPosition() )
+                seekTo = m_boundedPlayback->startPosition();
+            else if( seekTo > m_boundedPlayback->startPosition() + trackLength() )
+                seekTo = m_boundedPlayback->startPosition() + trackLength();
+        }
         else
             seekTo = ms;
 
@@ -736,9 +776,9 @@ EngineController::eqBandsFreq() const
     if( mEqPar.isEmpty() )
        return mBandsFreq;
     QRegExp rx( "\\d+(?=Hz)" );
-    foreach( Phonon::EffectParameter mParam, mEqPar )
+    foreach( const Phonon::EffectParameter &mParam, mEqPar )
     {
-    if( mParam.name().contains( QString( "pre-amp" ) ) )
+        if( mParam.name().contains( QString( "pre-amp" ) ) )
         {
             mBandsFreq << i18n( "Preamp" );
         }
@@ -778,7 +818,7 @@ EngineController::eqUpdate() //SLOT
 
         QListIterator<int> mEqParNewIt( mEqParCfg );
         double scaledVal; // Scaled value to set from universal -100 - 100 range to plugin scale
-        foreach( Phonon::EffectParameter mParam, mEqPar )
+        foreach( const Phonon::EffectParameter &mParam, mEqPar )
         {
             scaledVal = mEqParNewIt.hasNext() ? mEqParNewIt.next() : 0;
             scaledVal *= ( fabs(mParam.maximumValue().toDouble() ) +  fabs( mParam.minimumValue().toDouble() ) );
@@ -872,7 +912,7 @@ EngineController::slotAboutToFinish()
     else if ( m_currentTrack && m_currentTrack->playableUrl().url().startsWith( "audiocd:/" ) )
     {
         debug() << "finished a CD track, don't care if queue is not empty, just get new track...";
-        //m_media->stop();
+
         The::playlistActions()->requestNextTrack();
         slotQueueEnded();
     }
@@ -887,6 +927,7 @@ EngineController::slotQueueEnded()
 
     if( m_currentTrack && !m_multiPlayback && !m_multiSource )
     {
+        m_media->setCurrentSource( Phonon::MediaSource() );
         playbackEnded( trackPositionMs(), m_currentTrack->length(), EngineObserver::EndedStopped );
         m_currentTrack = 0;
         trackChangedNotify( m_currentTrack );
@@ -918,7 +959,12 @@ void
 EngineController::slotNewTrackPlaying( const Phonon::MediaSource &source )
 {
     DEBUG_BLOCK
-    Q_UNUSED( source );
+
+    if( source.type() == Phonon::MediaSource::Empty )
+    {
+        debug() << "Empty MediaSource (engine stop)";
+        return;
+    }
 
     // the new track was taken from the queue, so clear these fields
     if( m_nextTrack )
@@ -1135,6 +1181,7 @@ EngineController::slotStopFadeout() //SLOT
     DEBUG_BLOCK
 
     m_media->stop();
+    m_media->setCurrentSource( Phonon::MediaSource() );
     resetFadeout();
 }
 
@@ -1157,4 +1204,9 @@ void EngineController::slotTitleChanged( int titleNumber )
     slotAboutToFinish();
 }
 
+bool EngineController::isPlayingAudioCd()
+{
+    return m_currentIsAudioCd;
+}
+
 #include "EngineController.moc"
