@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2009 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                    *
+ * Copyright (c) 2009 Nikolaj Hald Nielsen <nhn@kde.org>                                *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -104,7 +104,7 @@ void BrowserCategory::setParentList( BrowserCategoryList * parent )
     m_parentList = parent;
 }
 
-BrowserCategoryList * BrowserCategory::parentList()
+BrowserCategoryList * BrowserCategory::parentList() const
 {
     return m_parentList;
 }
@@ -133,5 +133,27 @@ QString BrowserCategory::imagePath()
     return m_imagePath;
 }
 
+void
+BrowserCategory::addAdditionalItem( BrowserBreadcrumbItem * item )
+{
+    m_additionalItems.append( item );
+}
+
+void
+BrowserCategory::clearAdditionalItems()
+{
+    //these are deleted in BrowserBreadcrumbWidget::clearCrumbs
+    foreach( BrowserBreadcrumbItem * item, m_additionalItems )
+    {
+        m_additionalItems.removeAll( item );
+        delete item;
+    }
+}
+
+QList<BrowserBreadcrumbItem *>
+BrowserCategory::additionalItems()
+{
+    return m_additionalItems;
+}
 
 #include "BrowserCategory.moc"
