@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2008 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                    *
+ * Copyright (c) 2008 Nikolaj Hald Nielsen <nhn@kde.org>                                *
  * Copyright (r) 2009 Bart Cerneels <bart.cerneels@kde.org>                             *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
@@ -21,6 +21,8 @@
 #include "PaletteHandler.h"
 #include "playlist/PlaylistModel.h"
 #include "PlaylistsInGroupsProxy.h"
+#include "PlaylistsByProviderProxy.h"
+#include "PlaylistTreeItemDelegate.h"
 #include "SvgHandler.h"
 #include "statusbar/StatusBar.h"
 #include "UserPlaylistModel.h"
@@ -38,7 +40,12 @@
 
 #include <typeinfo>
 
-PlaylistBrowserNS::PlaylistCategory::PlaylistCategory( QWidget * parent )
+using namespace PlaylistBrowserNS;
+
+QString PlaylistCategory::s_configGroup( "Saved Playlists View" );
+QString PlaylistCategory::s_mergeViewKey( "Merged View" );
+
+PlaylistCategory::PlaylistCategory( QWidget * parent )
     : BrowserCategory( "user playlists", parent )
 {
     setPrettyName( i18n( "Saved Playlists" ) );
@@ -57,9 +64,23 @@ PlaylistBrowserNS::PlaylistCategory::PlaylistCategory( QWidget * parent )
     m_toolBar = new QToolBar( this );
     m_toolBar->setToolButtonStyle( Qt::ToolButtonTextBesideIcon );
 
-    m_groupedProxy = new PlaylistsInGroupsProxy( The::userPlaylistModel() );
+    KAction *toggleAction = new KAction( KIcon( "view-list-tree" ),
+                                         i18n( "Merged View" ), m_toolBar );
+    toggleAction->setCheckable( true );
+    toggleAction->setChecked( Amarok::config( s_configGroup ).readEntry( s_mergeViewKey, false ) );
+    m_toolBar->addAction( toggleAction );
+    connect( toggleAction, SIGNAL( triggered( bool ) ), SLOT( toggleView( bool ) ) );
+
+    m_playlistView = new UserPlaylistTreeView( The::userPlaylistModel(), this );
+    m_byProviderProxy = new PlaylistsByProviderProxy( The::userPlaylistModel(),
+                                                      UserModel::ProviderColumn );
+    m_byProviderDelegate = new PlaylistTreeItemDelegate( m_playlistView );
+
+    m_byFolderProxy = new PlaylistsInGroupsProxy( The::userPlaylistModel() );
+    m_defaultItemView = m_playlistView->itemDelegate();
+
+    toggleView( toggleAction->isChecked() );
 
-    m_playlistView = new UserPlaylistTreeView( m_groupedProxy, this );
 //    m_playlistView = new UserPlaylistTreeView( The::userPlaylistModel(), this );
     m_playlistView->setFrameShape( QFrame::NoFrame );
     m_playlistView->setContentsMargins( 0, 0, 0, 0 );
@@ -83,17 +104,37 @@ PlaylistBrowserNS::PlaylistCategory::PlaylistCategory( QWidget * parent )
              m_playlistView, SLOT( createNewGroup() ) );
 
     m_playlistView->setNewGroupAction( m_addGroupAction );
+    new PlaylistTreeItemDelegate( m_playlistView );
 }
 
-PlaylistBrowserNS::PlaylistCategory::~PlaylistCategory()
+PlaylistCategory::~PlaylistCategory()
 {
 }
 
-void PlaylistBrowserNS::PlaylistCategory::newPalette(const QPalette & palette)
+void PlaylistCategory::newPalette(const QPalette & palette)
 {
     Q_UNUSED( palette )
 
     The::paletteHandler()->updateItemView( m_playlistView );
 }
 
+void
+PlaylistCategory::toggleView( bool merged )
+{
+    if( merged )
+    {
+        m_playlistView->setModel( m_byFolderProxy );
+        m_playlistView->setItemDelegate( m_defaultItemView );
+        m_playlistView->setRootIsDecorated( true );
+    }
+    else
+    {
+        m_playlistView->setModel( m_byProviderProxy );
+        m_playlistView->setItemDelegate( m_byProviderDelegate );
+        m_playlistView->setRootIsDecorated( false );
+    }
+
+    Amarok::config( s_configGroup ).writeEntry( s_mergeViewKey, merged );
+}
+
 #include "PlaylistCategory.moc"
