@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2007 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                    *
+ * Copyright (c) 2007 Nikolaj Hald Nielsen <nhn@kde.org>                                *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -38,7 +38,7 @@ ServiceBrowser * ServiceBrowser::instance()
 
 
 ServiceBrowser::ServiceBrowser( QWidget * parent, const QString& name )
-    : BrowserCategoryList( parent, name )
+    : BrowserCategoryList( parent, name, true )
     , m_usingContextView( false )
 {
     debug() << "ServiceBrowser starting...";
