@@ -46,20 +46,22 @@
 #include <QDir>
 #include <QTextCodec>
 
-AMAROK_EXPORT_PLUGIN( AudioCdCollectionFactory )
+AMAROK_EXPORT_COLLECTION( AudioCdCollectionFactory, audiocdcollection )
 
 using namespace Meta;
 
-AudioCdCollectionFactory::AudioCdCollectionFactory()
+AudioCdCollectionFactory::AudioCdCollectionFactory( QObject *parent, const QVariantList &args )
     : MediaDeviceCollectionFactory<AudioCdCollection>( new AudioCdConnectionAssistant() )
 {
-    DEBUG_BLOCK
+    setParent( parent );
+    Q_UNUSED( args );
 }
 
 
 AudioCdCollection::AudioCdCollection( MediaDeviceInfo* info )
    : MediaDeviceCollection()
    , m_encodingFormat( OGG )
+   , m_ready( false )
 {
     DEBUG_BLOCK
 
@@ -271,6 +273,15 @@ AudioCdCollection::infoFetchComplete( KJob *job )
 
     emit ( updated() );
     updateProxyTracks();
+
+    m_ready = true;
+
+    //be nice and check if MainWindow is just aching for an audio cd to start playing
+    if( The::mainWindow()->isWaitingForCd() )
+    {
+        debug() << "Tell MainWindow to start playing us immediately.";
+        The::mainWindow()->playAudioCd();
+    }
 }
 
 QueryMaker *
@@ -462,6 +473,8 @@ AudioCdCollection::noInfoAvailable()
     emit ( updated() );
     updateProxyTracks();
 
+    m_ready = true;
+
 }
 
 void
@@ -535,25 +548,25 @@ AudioCdCollection::trackForUrl( const KUrl & url )
 void
 AudioCdCollection::updateProxyTracks()
 {
-    foreach( KUrl url, m_proxyMap.keys() )
+    foreach( const KUrl &url, m_proxyMap.keys() )
     {
 
-        QString urlString = url.url().remove( "audiocd:/" );
-        QStringList parts = urlString.split( '/' );
+        const QString &urlString = url.url().remove( "audiocd:/" );
+        const QStringList &parts = urlString.split( '/' );
 
-        if ( parts.count() != 2 )
+        if( parts.count() != 2 )
             continue;
 
-        QString discId = parts.at( 0 );
+        const QString &discId = parts.at( 0 );
 
-        if ( discId != m_discCddbId )
+        if( discId != m_discCddbId )
             continue;
 
-        int trackNumber = parts.at( 1 ).toInt();
+        const int trackNumber = parts.at( 1 ).toInt();
 
-        foreach( TrackPtr track, trackMap().values() )
+        foreach( const TrackPtr &track, trackMap().values() )
         {
-            if ( track->trackNumber() == trackNumber )
+            if( track->trackNumber() == trackNumber )
             {
                 m_proxyMap.value( url )->updateTrack( track );
             }
@@ -571,6 +584,11 @@ void AudioCdCollection::startFullScan()
     emit collectionReady( this );
 }
 
+bool AudioCdCollection::isReady()
+{
+    return m_ready;
+}
+
 
 
 #include "AudioCdCollection.moc"
