@@ -150,6 +150,17 @@ DaapTrack::setYear( const QString &newYear )
     Q_UNUSED( newYear )
 }
 
+/* 
+TODO: This isn't good enough, but for now as daapreader/Reader.cpp indicates
+ we can query for the BPM from daap server, but desire is to get BPM of files working
+ first! 
+*/
+float
+DaapTrack::bpm() const
+{
+    return -1.0;
+}
+
 QString
 DaapTrack::comment() const
 {
