@@ -8,7 +8,7 @@
  *                                                                                      *
  * This program is distributed in the hope that it will be useful, but WITHOUT ANY      *
  * WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A      *
- * PARTICULAR PURPOSE. See the GNU General Public License for more details.              *
+ * PARTICULAR PURPOSE. See the GNU General Public License for more details.             *
  *                                                                                      *
  * You should have received a copy of the GNU General Public License along with         *
  * this program.  If not, see <http://www.gnu.org/licenses/>.                           *
@@ -25,12 +25,15 @@
 #include "Debug.h"
 
 #include <KUrl>
+#include <KStandardDirs>
 
-AMAROK_EXPORT_PLUGIN( IpodCollectionFactory )
+AMAROK_EXPORT_COLLECTION( IpodCollectionFactory, ipodcollection )
 
-IpodCollectionFactory::IpodCollectionFactory()
+IpodCollectionFactory::IpodCollectionFactory( QObject *parent, const QVariantList &args )
     : MediaDeviceCollectionFactory<IpodCollection> ( new IpodConnectionAssistant() )
 {
+    setParent( parent );
+    Q_UNUSED( args )
 }
 
 IpodCollectionFactory::~IpodCollectionFactory()
@@ -39,18 +42,38 @@ IpodCollectionFactory::~IpodCollectionFactory()
 
 //IpodCollection
 
-IpodCollection::IpodCollection(MediaDeviceInfo* info)
+IpodCollection::IpodCollection( MediaDeviceInfo* info )
     : MediaDeviceCollection()
 {
     DEBUG_BLOCK
     /** Fetch Info needed to construct IpodCollection */
     IpodDeviceInfo *ipodinfo = qobject_cast<IpodDeviceInfo *>( info );
 
-    m_mountPoint = ipodinfo->mountpoint();
+    m_mountPoint = ipodinfo->mountPoint();
     debug() << "Mounted at: " << m_mountPoint;
     m_udi = ipodinfo->udi();
 
-    m_handler = new Meta::IpodHandler( this, m_mountPoint );
+    bool isMounted = true;
+    if (m_mountPoint.isEmpty())
+    {
+        isMounted = false;
+        m_mountPoint = KStandardDirs::locateLocal( "data", "amarok/tmp/" );
+        m_mountPoint += "imobiledevice";
+        if( !ipodinfo->deviceUid().isEmpty() )
+            m_mountPoint += '_' + ipodinfo->deviceUid();
+        debug() << "set mountpoint to " << m_mountPoint;
+
+        QDir mp(m_mountPoint);
+        if(!mp.exists())
+        {
+            mp.mkpath(m_mountPoint);
+            debug() << "created " << m_mountPoint;
+        }
+
+        ipodinfo->setMountPoint( m_mountPoint );
+    }
+
+    m_handler = new Meta::IpodHandler( this, ipodinfo );
 }
 
 
