@@ -69,7 +69,7 @@ using namespace Meta;
 
 /// IpodHandler
 
-IpodHandler::IpodHandler( IpodCollection *mc, const QString& mountPoint )
+IpodHandler::IpodHandler( IpodCollection *mc, const IpodDeviceInfo *deviceInfo )
     : MediaDeviceHandler( mc )
     , m_itdb( 0 )
     , m_masterPlaylist( 0 )
@@ -84,11 +84,9 @@ IpodHandler::IpodHandler( IpodCollection *mc, const QString& mountPoint )
     , m_jobcounter( 0 )
     , m_libtrack( 0 )
     , m_autoConnect( false )
-    , m_mountPoint( mountPoint )
     , m_name()
+    , m_deviceInfo( deviceInfo )
     , m_isShuffle( false )
-    , m_isMobile( false )
-    , m_isIPhone( false )
     , m_supportsArtwork( false )
     , m_supportsVideo( false )
     , m_rockboxFirmware( false )
@@ -117,18 +115,108 @@ IpodHandler::~IpodHandler()
     writeITunesDB( false );
     if ( m_itdb )
         itdb_free( m_itdb );
+
+    if( m_autoConnect )
+    {
+        QProcess unmount;
+        QStringList args;
+        args << "-u" << "-z" << mountPoint();
+        unmount.start("fusermount", args);
+        bool ok = unmount.waitForStarted();
+        if( !ok )
+        {
+            debug() << "fusermount for unmounting" << mountPoint() << "failed to start";
+        }
+        else
+        {
+            ok = unmount.waitForFinished();
+            if( !ok )
+                debug() << "fusermount did not terminate correctly";
+        }
+        if( ok )
+        {
+            ok = unmount.exitStatus() == QProcess::NormalExit;
+            if( !ok )
+                debug() << "fusermount did not exit normally";
+        }
+        if( ok )
+        {
+            ok = unmount.exitCode() == 0;
+            if( !ok )
+                debug() << "fusermount did not exit successfully";
+        }
+        if( ok )
+        {
+            debug() << "Unmounted imobiledevice using ifuse from" << mountPoint();
+        }
+        else
+        {
+            debug() << "Unmounting imobiledevice using ifuse from" << mountPoint() << "failed";
+        }
+    }
 }
 
 void
 IpodHandler::init()
 {
-    if( m_mountPoint.isEmpty() )
+    bool isMounted = m_deviceInfo->wasMounted();
+    if ( !isMounted )
+    {
+        QStringList args;
+        if( !m_deviceInfo->deviceUid().isEmpty() )
+        {
+            args << "--uuid";
+            args << m_deviceInfo->deviceUid();
+            args << QString("-ofsname=afc://%1").arg(m_deviceInfo->deviceUid());
+        }
+        args << mountPoint();
+        QProcess ifuse;
+        debug() << "calling ifuse with args" << args;
+        ifuse.start("ifuse", args);
+        bool ok = ifuse.waitForStarted();
+        if( !ok )
+        {
+            debug() << "Failed to start ifuse";
+        }
+        else
+        {
+            ok = ifuse.waitForFinished();
+            if( !ok )
+                debug() << "ifuse did not yet terminate";
+        }
+        if( ok )
+        {
+            ok = ifuse.exitStatus() == QProcess::NormalExit;
+            if( !ok )
+                debug() << "ifuse crashed";
+        }
+        if( ok )
+        {
+            ok = ifuse.exitCode() == 0;
+            if( !ok )
+                debug() << "ifuse exited with non-zero exit code";
+        }
+
+        if( ok )
+        {
+            isMounted = true;
+            debug() << "Successfully mounted imobiledevice using ifuse on" << mountPoint();
+        }
+        else
+        {
+            debug() << "Mounting imobiledevice using ifuse on" << mountPoint() << "failed";
+        }
+    }
+
+    if( !isMounted )
     {
         debug() << "Error: empty mountpoint, probably an unmounted iPod, aborting";
         m_memColl->slotAttemptConnectionDone( false );
         return;
     }
 
+    m_autoConnect = !m_deviceInfo->wasMounted();
+
     GError *err = 0;
     QString initError = i18n( "iPod was not initialized:" );
     QString initErrorCaption = i18n( "iPod Initialization Failed" );
@@ -137,7 +225,7 @@ IpodHandler::init()
     // First attempt to parse the database
 
     debug() << "Calling the db parser";
-    m_itdb = itdb_parse( QFile::encodeName( m_mountPoint ),  &err );
+    m_itdb = itdb_parse( QFile::encodeName( mountPoint() ),  &err );
 
     // If this fails, we will ask the user if he wants to init the device
 
@@ -317,16 +405,23 @@ IpodHandler::init()
     Solid::Device device = Solid::Device( m_memColl->udi() );
     if( device.isValid() )
     {
-        Solid::StorageAccess *storage = device.as<Solid::StorageAccess>();
-        m_filepath = storage->filePath();
-        m_capacity = KDiskFreeSpaceInfo::freeSpaceInfo( m_filepath ).size();
+        if (Solid::StorageAccess *storage = device.as<Solid::StorageAccess>())
+            m_filepath = storage->filePath();
+        else if (!mountPoint().isEmpty())
+            m_filepath = mountPoint();
+        else
+            m_filepath.clear();
     }
     else
     {
-        m_filepath = "";
-        m_capacity = 0.0;
+        m_filepath.clear();
     }
 
+    if( m_filepath.isEmpty() )
+        m_capacity = 0.0;
+    else
+        m_capacity = KDiskFreeSpaceInfo::freeSpaceInfo( m_filepath ).size();
+
     debug() << "Succeeded: " << m_success;
 
     m_memColl->slotAttemptConnectionDone( m_success );
@@ -501,7 +596,7 @@ IpodHandler::syncArtwork()
 
     Meta::AlbumMap albumMap = m_memColl->albumMap();
 
-    foreach( Meta::AlbumPtr album, albumMap.values() )
+    foreach( const Meta::AlbumPtr album, albumMap.values() )
     {
         debug() << "Updating artwork for" << (album->albumArtist() ? album->albumArtist()->name() : "unknown") << "-" << album->name();
     }
@@ -570,30 +665,19 @@ IpodHandler::initializeIpod()
     itdb_playlist_add(m_itdb, podcasts, -1);
     itdb_playlist_add(m_itdb, mpl, 0);
 
-    QString realPath;
-    if(!pathExists( itunesDir(), &realPath) )
-    {
-        dir.setPath( realPath );
-        dir.mkdir( dir.absolutePath() );
-    }
-    if( !dir.exists() )
-        return false;
+    QStringList dirs;
+    dirs << itdb_get_control_dir(itdb_get_mountpoint(m_itdb));
+    dirs << itdb_get_music_dir(itdb_get_mountpoint(m_itdb));
+    dirs << itdb_get_itunes_dir(itdb_get_mountpoint(m_itdb));
 
-    if( !pathExists( itunesDir( "Music" ), &realPath ) )
+    for(QStringList::iterator it = dirs.begin(); it != dirs.end(); ++it)
     {
-        dir.setPath( realPath );
-        dir.mkdir( dir.absolutePath() );
+        dir.setPath(*it);
+        if( !dir.exists() )
+            dir.mkdir( dir.absolutePath() );
+        if( !dir.exists() )
+            return false;
     }
-    if( !dir.exists() )
-        return false;
-
-    if( !pathExists( itunesDir( "iTunes" ), &realPath ) )
-    {
-        dir.setPath( realPath );
-        dir.mkdir( dir.absolutePath() );
-    }
-    if( !dir.exists() )
-        return false;
 
     m_dbChanged = true;
 
@@ -617,12 +701,9 @@ IpodHandler::detectModel()
     m_supportsArtwork = true;
 
     m_supportsVideo = false;
-    m_isIPhone = false;
     m_needsFirewireGuid = false;
     m_rockboxFirmware = false;
 
-    // needs recent libgpod-0.3.3 from cvs
-    bool guess = false;
     if( m_itdb && m_itdb->device )
     {
         debug() << "Attempting to get info...";
@@ -635,6 +716,10 @@ IpodHandler::detectModel()
         m_supportsArtwork = false;
         #endif
         debug() << "Supports Artwork: " << ( m_supportsArtwork ? "true" : "false" );
+
+        m_supportsVideo = itdb_device_supports_video( m_itdb->device );
+        debug() << "Supports Video: " << ( m_supportsVideo ? "true" : "false" );
+
         QString musicdirs;
         musicdirs.setNum( itdb_musicdirs_number(m_itdb) );
         debug() << "Musicdirs: " << musicdirs;
@@ -659,33 +744,9 @@ IpodHandler::detectModel()
                 m_isShuffle = true;
                 break;
 
-            case ITDB_IPOD_MODEL_IPHONE_1:
-            //TODO newer libgpod thinks that an iPod touch is silver, older that it's black
-            //case ITDB_IPOD_MODEL_TOUCH_BLACK:
-            //case ITDB_IPOD_MODEL_TOUCH_SILVER:
-                m_isIPhone = true;
-                debug() << "detected iPhone/iPod Touch" << endl;
-                break;
-
-            case ITDB_IPOD_MODEL_CLASSIC_SILVER:
-            case ITDB_IPOD_MODEL_CLASSIC_BLACK:
-                debug() << "detected iPod classic";
-            case ITDB_IPOD_MODEL_VIDEO_WHITE:
-            case ITDB_IPOD_MODEL_VIDEO_BLACK:
-            case ITDB_IPOD_MODEL_VIDEO_U2:
-                m_supportsVideo = true;
-                debug() << "detected video-capable iPod";
-                break;
-
-            case ITDB_IPOD_MODEL_MOBILE_1:
-                m_isMobile = true;
-                debug() << "detected iTunes phone" << endl;
-                break;
-
             case ITDB_IPOD_MODEL_INVALID:
             case ITDB_IPOD_MODEL_UNKNOWN:
                 modelString = 0;
-                guess = true;
                 break;
 
             default:
@@ -700,11 +761,6 @@ IpodHandler::detectModel()
                case ITDB_IPOD_GENERATION_NANO_3:
                case ITDB_IPOD_GENERATION_TOUCH_1:
                   m_needsFirewireGuid = true;
-                  m_supportsVideo = true;
-                  break;
-               case ITDB_IPOD_GENERATION_VIDEO_1:
-               case ITDB_IPOD_GENERATION_VIDEO_2:
-                  m_supportsVideo = true;
                   break;
                case ITDB_IPOD_GENERATION_SHUFFLE_1:
                case ITDB_IPOD_GENERATION_SHUFFLE_2:
@@ -718,43 +774,11 @@ IpodHandler::detectModel()
         }
         if( modelString )
             m_name = QString( "iPod %1" ).arg( QString::fromUtf8( modelString ) );
-
-        if( m_needsFirewireGuid )
-        {
-            gchar *fwid = itdb_device_get_sysinfo( m_itdb->device, "FirewireGuid" );
-            if( fwid )
-               g_free( fwid );
-        }
     }
     else
     {
         debug() << "iPod type detection failed, no video support";
         m_needsFirewireGuid = true; // can't read db because no firewire, maybe
-        guess = true;
-    }
-
-    if( guess )
-    {
-        if( pathExists( ":iTunes:iTunes_Control" ) )
-        {
-            debug() << "iTunes/iTunes_Control found - assuming itunes phone" << endl;
-            m_isMobile = true;
-        }
-        else if( pathExists( ":iTunes_Control" ) )
-        {
-            debug() << "iTunes_Control found - assuming iPhone/iPod Touch" << endl;
-            m_isIPhone = true;
-        }
-    }
-
-    if( m_isIPhone )
-    {
-        #ifdef GDK_FOUND
-        m_supportsArtwork = true;
-        #else
-        m_supportsArtwork = false;
-        #endif
-        m_supportsVideo = true;
     }
 
     if( pathExists( ":.rockbox" ) )
@@ -1003,13 +1027,16 @@ IpodHandler::writeITunesDB( bool threaded )
 QString
 IpodHandler::itunesDir(const QString &p) const
 {
-    QString base( ":iPod_Control" );
-    if( m_isMobile )
-        base = ":iTunes:iTunes_Control";
+    if( m_controlDir.isEmpty() )
+    {
+        const QString realControlDir= itdb_get_control_dir( itdb_get_mountpoint( m_itdb ) );
+        m_controlDir = ipodPath( realControlDir );
+    }
 
-    if( !p.startsWith( ':' ) )
-        base += ':';
-    return base + p;
+    if( p.startsWith( ':' ) )
+        return m_controlDir + p;
+    else
+        return m_controlDir + ':' + p;
 }
 
 /// Finds path to copy track to on Ipod
@@ -1156,7 +1183,7 @@ IpodHandler::staleTracks()
 
     Meta::TrackMap trackMap = m_memColl->trackMap();
 
-    foreach( Meta::TrackPtr trk, trackMap.values() )
+    foreach( const Meta::TrackPtr trk, trackMap.values() )
     {
         Meta::MediaDeviceTrackPtr track = Meta::MediaDeviceTrackPtr::staticCast( trk );
         Itdb_Track *ipodtrack =  m_itdbtrackhash.value( track );
@@ -1177,12 +1204,7 @@ IpodHandler::orphanedTracks()
     DEBUG_BLOCK
 
     QStringList orphanedTracks;
-    QString musicpath;
-    if (!pathExists( itunesDir( "Music" ), &musicpath ))
-    {
-        debug() << "Music path not found";
-        return QStringList();
-    }
+    QString musicpath = itdb_get_music_dir( itdb_get_mountpoint( m_itdb ) );
 
     debug() << "Found path for Music";
 
@@ -1375,7 +1397,7 @@ IpodHandler::determineURLOnDevice( const Meta::TrackPtr &track )
         int dir = num % music_dirs;
         QString dirname;
         debug() << "itunesDir(): " << itunesDir();
-        dirname = QString( "%1Music:F%2" ).arg( "iPod_Control:" ).arg( QString::number( dir, 10 ), 2, QLatin1Char( '0' ) );
+        dirname = itunesDir( QString( "Music:F%1" ).arg( QString::number( dir, 10 ), 2, QLatin1Char( '0' ) ) );
 
         debug() << "Copying to dirname: " << dirname;
         if( !pathExists( dirname ) )
@@ -1394,7 +1416,7 @@ IpodHandler::determineURLOnDevice( const Meta::TrackPtr &track )
 }
 
 QString
-IpodHandler::ipodPath( const QString &realPath )
+IpodHandler::ipodPath( const QString &realPath ) const
 {
     if( m_itdb )
     {
@@ -1531,7 +1553,7 @@ IpodHandler::libGetType( const Meta::MediaDeviceTrackPtr &track )
 KUrl
 IpodHandler::libGetPlayableUrl( const Meta::MediaDeviceTrackPtr &track )
 {
-    return KUrl(m_mountPoint + (QString( m_itdbtrackhash[ track ]->ipod_path ).split( ':' ).join( "/" )));
+    return KUrl(mountPoint() + (QString( m_itdbtrackhash[ track ]->ipod_path ).split( ':' ).join( "/" )));
 }
 
 float
@@ -1602,7 +1624,7 @@ IpodHandler::libSetYear( Meta::MediaDeviceTrackPtr &track, const QString& year )
 void
 IpodHandler::libSetLength( Meta::MediaDeviceTrackPtr &track, int length )
 {
-    m_itdbtrackhash[ track ]->tracklen = length*1000;
+    m_itdbtrackhash[ track ]->tracklen = length;
     setDatabaseChanged();
 }
 void
@@ -1997,7 +2019,7 @@ IpodHandler::setAssociateTrack( const Meta::MediaDeviceTrackPtr track )
 QStringList
 IpodHandler::supportedFormats()
 {
-    return ( QStringList() ) << "mp3" << "aac" << "mp4" << "m4a" << "aiff";
+    return ( QStringList() ) << "mp3" << "aac" << "mp4" << "m4a" << "aiff" << "m4b" << "wav" << "mpeg" << "m4v" << "mpv4" << "mov";
 }
 
 
@@ -2068,7 +2090,7 @@ IpodHandler::slotStaleSucceeded( ThreadWeaver::Job* job )
         if( m_staletracks.count() > 0 )
         {
 
-            foreach( Meta::TrackPtr track, m_staletracks )
+            foreach( const Meta::TrackPtr track, m_staletracks )
             {
                 QString ent;
                 QTextStream entry( &ent );
@@ -2088,7 +2110,7 @@ IpodHandler::slotStaleSucceeded( ThreadWeaver::Job* job )
             if( ok )
             {
                 Meta::TrackList staleToDelete;
-                foreach( QString item, itemList )
+                foreach( const QString &item, itemList )
                 {
                     staleToDelete << m_staletracks[ itemList.indexOf( item ) ];
                 }
