@@ -52,6 +52,7 @@ class EditCapabilityMediaDevice : public Meta::EditCapability
         virtual void setComposer( const QString &newComposer ) { m_track->setComposer( newComposer ); }
         virtual void setGenre( const QString &newGenre ) { m_track->setGenre( newGenre ); }
         virtual void setYear( const QString &newYear ) { m_track->setYear( newYear ); }
+        virtual void setBpm( const float newBpm ) { m_track->setBpm( newBpm ); }
         virtual void setTitle( const QString &newTitle ) { m_track->setTitle( newTitle ); }
         virtual void setComment( const QString &newComment ) { m_track->setComment( newComment ); }
         virtual void setTrackNumber( int newTrackNumber ) { m_track->setTrackNumber( newTrackNumber ); }
@@ -274,7 +275,7 @@ MediaDeviceTrack::bpm() const
     return m_bpm;
 }
 void
-MediaDeviceTrack::setBpm( float newBpm )
+MediaDeviceTrack::setBpm( const float newBpm )
 {
     m_bpm = newBpm;
 }
