@@ -309,6 +309,7 @@ MediaDeviceHandler::copyTrackListToDevice(const Meta::TrackList tracklist)
     DEBUG_BLOCK
 
     setupWriteCapability();
+    setupReadCapability();
 
     if( !m_wcb )
         return;
@@ -454,6 +455,7 @@ MediaDeviceHandler::copyTrackListToDevice(const Meta::TrackList tracklist)
 void
 MediaDeviceHandler::copyNextTrackToDevice()
 {
+    DEBUG_BLOCK
     Meta::TrackPtr track;
 
     // If there are more tracks to copy, copy the next one
@@ -810,14 +812,14 @@ MediaDeviceHandler::setupYearMap( Meta::MediaDeviceTrackPtr track, YearMap& year
     track->setYear( yearPtr );
 }
 
-void
-MediaDeviceHandler::parseTracks()
+bool
+MediaDeviceHandler::privateParseTracks()
 {
     DEBUG_BLOCK
 
     setupReadCapability();
     if ( !m_rcb )
-        return;
+        return false;
 
     TrackMap trackMap;
     ArtistMap artistMap;
@@ -930,6 +932,11 @@ MediaDeviceHandler::parseTracks()
 
     }
 
+    if( !m_podcastCapability && hasCapabilityInterface( Handler::Capability::Podcast ) )
+    {
+
+    }
+
     // Finally, assign the created maps to the collection
     m_memColl->acquireWriteLock();
     m_memColl->setTrackMap( trackMap );
@@ -941,6 +948,8 @@ MediaDeviceHandler::parseTracks()
     m_memColl->releaseLock();
 
     m_memColl->collectionUpdated();
+
+    return true;
 }
 
 void
@@ -1002,15 +1011,22 @@ MediaDeviceHandler::slotCopyTrackJobsDone( ThreadWeaver::Job* job )
 float
 MediaDeviceHandler::freeSpace() const
 {
+    DEBUG_BLOCK
     if ( m_rcb )
+    {
+        debug() << "totalCapacity:" << m_rcb->totalCapacity();
+        debug() << "usedCapacity():" << m_rcb->usedCapacity();
         return ( m_rcb->totalCapacity() - m_rcb->usedCapacity() );
-    else
+    } else {
+        debug() << "m_rcb null!";
         return 0.0;
+    }
 }
 
 float
 MediaDeviceHandler::usedcapacity() const
 {
+    DEBUG_BLOCK
     if ( m_rcb )
         return m_rcb->usedCapacity();
     else
@@ -1020,6 +1036,7 @@ MediaDeviceHandler::usedcapacity() const
 float
 MediaDeviceHandler::totalcapacity() const
 {
+    DEBUG_BLOCK
     if ( m_rcb )
         return m_rcb->totalCapacity();
     else
@@ -1227,6 +1244,47 @@ MediaDeviceHandler::metadataChanged( YearPtr year )
     Q_UNUSED( year );
 }
 
+void
+MediaDeviceHandler::parseTracks()
+{
+    ThreadWeaver::Weaver::instance()->enqueue( new ParseWorkerThread( this ) );
+}
+
+// ParseWorkerThread
+
+ParseWorkerThread::ParseWorkerThread( MediaDeviceHandler* handler )
+        : ThreadWeaver::Job()
+        , m_success( false )
+        , m_handler( handler )
+{
+    connect( this, SIGNAL( done( ThreadWeaver::Job* ) ), this, SLOT( slotDoneSuccess(ThreadWeaver::Job*)) );
+}
+
+ParseWorkerThread::~ParseWorkerThread()
+{
+    //nothing to do
+}
+
+bool
+ParseWorkerThread::success() const
+{
+    return m_success;
+}
+
+void
+ParseWorkerThread::run()
+{
+    m_success = m_handler->privateParseTracks();
+}
+
+void
+ParseWorkerThread::slotDoneSuccess( ThreadWeaver::Job* )
+{
+    m_handler->m_memColl->emitCollectionReady();
+}
+
+// CopyWorkerThread
+
 CopyWorkerThread::CopyWorkerThread( const Meta::TrackPtr &track, MediaDeviceHandler* handler )
         : ThreadWeaver::Job()
         , m_success( false )
