@@ -20,6 +20,7 @@
 #define PROXYCOLLECTION_H
 
 #include "collection/Collection.h"
+#include "collection/CollectionManager.h"
 #include "meta/Meta.h"
 #include "meta/MetaUtility.h"
 
@@ -40,7 +41,7 @@ namespace ProxyCollection
 
 
 
-    class Collection : public Amarok::Collection
+    class AMAROK_EXPORT_TESTS Collection : public Amarok::Collection
     {
         Q_OBJECT
         public:
@@ -94,10 +95,10 @@ namespace ProxyCollection
         public slots:
         void removeCollection( const QString &collectionId );
         void removeCollection( Amarok::Collection *collection );
-        void addCollection( Amarok::Collection *collection );
+        void addCollection( Amarok::Collection *collection, CollectionManager::CollectionStatus status );
+        void slotUpdated();
 
         private slots:
-        void slotUpdated();
         void emptyCache();
 
         private:
