@@ -17,33 +17,40 @@
 
 #include "SqlCollectionLocation.h"
 
-#include "AFTUtility.h"
+#include "collectionscanner/AFTUtility.h"
+
+#include "collection/CollectionLocationDelegate.h"
+#include "Components.h"
 #include "Debug.h"
-#include "Meta.h"
-#include "MetaUtility.h"
+#include "collection/SqlStorage.h"
+#include "meta/Meta.h"
+#include "meta/MetaUtility.h"
 #include "MountPointManager.h"
 #include "dialogs/OrganizeCollectionDialog.h"
 #include "ScanManager.h"
 #include "ScanResultProcessor.h"
 #include "SqlCollection.h"
 #include "SqlMeta.h"
-#include "../../statusbar/StatusBar.h"
+#include "statusbar/StatusBar.h"
 
 #include <QDir>
 #include <QFile>
 #include <QFileInfo>
 
+#include <kdiskfreespaceinfo.h>
 #include <kjob.h>
 #include <KLocale>
 #include <KSharedPtr>
 #include <kio/job.h>
 #include <kio/jobclasses.h>
+#include <kio/deletejob.h>
+#include <KMessageBox>
 
 
 using namespace Meta;
 
 SqlCollectionLocation::SqlCollectionLocation( SqlCollection const *collection )
-    : CollectionLocation()
+    : CollectionLocation( collection )
     , m_collection( const_cast<SqlCollection*>( collection ) )
     , m_overwriteFiles( false )
 {
@@ -64,12 +71,34 @@ SqlCollectionLocation::prettyLocation() const
 QStringList
 SqlCollectionLocation::actualLocation() const
 {
-    return MountPointManager::instance()->collectionFolders();
+    return m_collection->mountPointManager()->collectionFolders();
 }
+
 bool
 SqlCollectionLocation::isWritable() const
 {
-    return true;
+    // The collection is writeable if there exists a path that has less than
+    // 95% free space.
+    bool path_exists_with_space = false;
+    bool path_exists_writeable = false;
+    QStringList folders = actualLocation();
+    foreach(QString path, folders)
+    {
+        float used = KDiskFreeSpaceInfo::freeSpaceInfo( path ).used();
+        float total = KDiskFreeSpaceInfo::freeSpaceInfo( path ).size();
+
+        if( total <= 0 ) // protect against div by zero
+            continue; //How did this happen?
+
+        float percentage_used = used / total;
+        if( percentage_used < 0.95 )
+            path_exists_with_space = true;
+
+        QFileInfo info( path );
+        if( info.isWritable() )
+            path_exists_writeable = true;
+    }
+    return path_exists_with_space && path_exists_writeable;
 }
 
 bool
@@ -81,33 +110,21 @@ SqlCollectionLocation::isOrganizable() const
 bool
 SqlCollectionLocation::remove( const Meta::TrackPtr &track )
 {
+    DEBUG_BLOCK
     KSharedPtr<SqlTrack> sqlTrack = KSharedPtr<SqlTrack>::dynamicCast( track );
     if( sqlTrack && sqlTrack->inCollection() && sqlTrack->collection()->collectionId() == m_collection->collectionId() )
     {
+        debug() << "much much";
         bool removed;
-        //SqlCollectionLocation uses KIO::move for moving files internally
-        //therefore we check whether the destination CollectionLocation
-        //represents the same collection, and check the existence of the
-        //file. If it does not exist, we assume that it has been moved, and remove it from the database.
-        //at worst we get a warning about a file not in the database. If it still exists, and
-        //this method has been called, do not do anything, as something is wrong.
-        //If the destination location is another collection, remove the file as we expect
-        //the destination to tell us if it is really really sure that it has copied a file.
-        //If we are not copying/moving files destination() will be 0.
-        if( destination() && destination()->collection() == collection() )
-        {
-            removed = !QFile::exists( sqlTrack->playableUrl().path() );
-        }
-        else
-        {
-            removed = QFile::remove( sqlTrack->playableUrl().path() );
-        }
+        // we are going to delete it from the database only if is no longer on disk
+        removed = !QFile::exists( sqlTrack->playableUrl().path() );
+
         if( removed )
         {
 
             QString query = QString( "SELECT id FROM urls WHERE deviceid = %1 AND rpath = '%2';" )
-                                .arg( QString::number( sqlTrack->deviceid() ), m_collection->escape( sqlTrack->rpath() ) );
-            QStringList res = m_collection->query( query );
+                                .arg( QString::number( sqlTrack->deviceid() ), m_collection->sqlStorage()->escape( sqlTrack->rpath() ) );
+            QStringList res = m_collection->sqlStorage()->query( query );
             if( res.isEmpty() )
             {
                 warning() << "Tried to remove a track from SqlCollection which is not in the collection";
@@ -115,15 +132,15 @@ SqlCollectionLocation::remove( const Meta::TrackPtr &track )
             else
             {
                 int id = res[0].toInt();
-                QString query = QString( "DELETE FROM tracks where id = %1;" ).arg( id );
-                m_collection->query( query );
+                QString query = QString( "DELETE FROM tracks where url = %1;" ).arg( id );
+                m_collection->sqlStorage()->query( query );
             }
         }
         if( removed )
         {
             QFileInfo file( sqlTrack->playableUrl().path() );
             QDir dir = file.dir();
-            const QStringList collectionFolders = MountPointManager::instance()->collectionFolders();
+            const QStringList collectionFolders = m_collection->mountPointManager()->collectionFolders();
             while( !collectionFolders.contains( dir.absolutePath() ) && !dir.isRoot() && dir.count() == 0 )
             {
                 const QString name = dir.dirName();
@@ -137,6 +154,7 @@ SqlCollectionLocation::remove( const Meta::TrackPtr &track )
     }
     else
     {
+        debug() << "Remove Failed: track exists on disk." << sqlTrack->playableUrl().path();
         return false;
     }
 }
@@ -144,9 +162,61 @@ SqlCollectionLocation::remove( const Meta::TrackPtr &track )
 void
 SqlCollectionLocation::showDestinationDialog( const Meta::TrackList &tracks, bool removeSources )
 {
+    DEBUG_BLOCK
     setGoingToRemoveSources( removeSources );
+
+    KIO::filesize_t transferSize = 0;
+    foreach( Meta::TrackPtr track, tracks )
+        transferSize += track->filesize();
+
+    QStringList actual_folders = actualLocation(); // the folders in the collection
+    QStringList available_folders; // the folders which have freespace available
+    foreach(QString path, actual_folders)
+    {
+        if( path.isEmpty() )
+            continue;
+        debug() << "Path" << path;
+        KDiskFreeSpaceInfo spaceInfo = KDiskFreeSpaceInfo::freeSpaceInfo( path );
+        if( !spaceInfo.isValid() )
+            continue;
+
+        KIO::filesize_t totalCapacity = spaceInfo.size();
+        KIO::filesize_t used = spaceInfo.used();
+
+        KIO::filesize_t freeSpace = totalCapacity - used;
+
+        debug() << "used:" << used;
+        debug() << "total:" << totalCapacity;
+        debug() << "Free space" << freeSpace;
+        debug() << "transfersize" << transferSize;
+
+        if( totalCapacity <= 0 ) // protect against div by zero
+            continue; //How did this happen?
+
+        double percentageUsedAfter = double( used + transferSize ) / totalCapacity;
+        debug() << "percentage used after" << percentageUsedAfter;
+
+        QFileInfo info( path );
+
+        // since bad things happen when drives become totally full, we define full as 95% capacity used
+        // also we make sure there is at least 5 megabytes free
+        // finally, ensure the path is writeable
+        debug() << ( freeSpace - transferSize );
+        if( ( freeSpace - transferSize ) > 1024*1024*5 && ( percentageUsedAfter < 0.95 ) && info.isWritable() )
+            available_folders << path;
+    }
+
+    if( available_folders.size() <= 0 )
+    {
+        debug() << "No space available or not writable";
+        CollectionLocationDelegate *delegate = Amarok::Components::collectionLocationDelegate();
+        delegate->notWriteable( this );
+        abort();
+        return;
+    }
+
     OrganizeCollectionDialog *dialog = new OrganizeCollectionDialog( tracks,
-                MountPointManager::instance()->collectionFolders(),
+                available_folders,
                 The::mainWindow(), //parent
                 "", //name is unused
                 true, //modal
@@ -164,6 +234,16 @@ SqlCollectionLocation::slotDialogAccepted()
     OrganizeCollectionDialog *dialog = qobject_cast<OrganizeCollectionDialog*>( sender() );
     m_destinations = dialog->getDestinations();
     m_overwriteFiles = dialog->overwriteDestinations();
+    if( isGoingToRemoveSources() )
+    {
+        CollectionLocationDelegate *delegate = Amarok::Components::collectionLocationDelegate();
+        const bool del = delegate->reallyMove( this, m_destinations.keys() );
+        if( !del )
+        {
+            abort();
+            return;
+        }
+    }
     slotShowDestinationDialogDone();
 }
 
@@ -178,6 +258,7 @@ SqlCollectionLocation::slotDialogRejected()
 void
 SqlCollectionLocation::slotJobFinished( KJob *job )
 {
+    DEBUG_BLOCK
     if( job->error() )
     {
         //TODO: proper error handling
@@ -193,6 +274,14 @@ SqlCollectionLocation::slotJobFinished( KJob *job )
 
     if( !startNextJob() )
     {
+        // filter the list of destinations to only include tracks
+        // that were successfully copied
+        foreach( const Meta::TrackPtr &track, m_destinations.keys() )
+        {
+            if( !QFileInfo( m_destinations[ track ] ).exists() )
+                m_destinations.remove( track );
+        }
+        The::statusBar()->endProgressOperation( this );
         insertTracks( m_destinations );
         insertStatistics( m_destinations );
         m_collection->scanManager()->setBlockScan( false );
@@ -201,6 +290,37 @@ SqlCollectionLocation::slotJobFinished( KJob *job )
 
 }
 
+void
+SqlCollectionLocation::slotRemoveJobFinished( KJob *job )
+{
+    DEBUG_BLOCK
+    if( job->error() )
+    {
+        //TODO: proper error handling
+        warning() << "An error occurred when removing a file: " << job->errorString();
+        transferError(m_removejobs.value( job ), KIO::buildErrorString( job->error(), job->errorString() ) );
+    }
+    else
+    {
+        // Remove the track from the database
+        remove( m_removejobs.value( job ) );
+
+        //we  assume that KIO works correctly...
+        transferSuccessful( m_removejobs.value( job ) );
+    }
+
+    m_removejobs.remove( job );
+    job->deleteLater();
+
+    if( !startNextRemoveJob() )
+    {
+        m_collection->scanManager()->setBlockScan( false );
+//         collection()->collectionUpdated();
+        slotRemoveOperationFinished();
+    }
+
+}
+
 void
 SqlCollectionLocation::copyUrlsToCollection( const QMap<Meta::TrackPtr, KUrl> &sources )
 {
@@ -208,15 +328,41 @@ SqlCollectionLocation::copyUrlsToCollection( const QMap<Meta::TrackPtr, KUrl> &s
 
     m_sources = sources;
 
+    QString statusBarTxt;
+
+    if( destination() == source() )
+        statusBarTxt = i18n( "Organizing tracks" );
+    else if ( isGoingToRemoveSources() )
+        statusBarTxt = i18n( "Moving tracks" );
+    else
+        statusBarTxt = i18n( "Copying tracks" );
+
+    The::statusBar()->newProgressOperation( this, statusBarTxt );
+    The::statusBar()->incrementProgressTotalSteps( this, m_sources.size() );
+
     if( !startNextJob() ) //this signal needs to be called no matter what, even if there are no job finishes to call it
+    {
+        The::statusBar()->endProgressOperation( this );
+        m_collection->scanManager()->setBlockScan( false ); //unblock scanning if we encountered an error while copying as well
         slotCopyOperationFinished();
+    }
 }
 
 void
 SqlCollectionLocation::removeUrlsFromCollection(  const Meta::TrackList &sources )
 {
-    m_collection->deleteTracksSlot( sources );
-    slotRemoveOperationFinished();
+    DEBUG_BLOCK
+
+    m_collection->scanManager()->setBlockScan( true );  //make sure the collection scanner does not run while we are deleting stuff
+
+    m_removetracks = sources;
+
+    if( !startNextRemoveJob() ) //this signal needs to be called no matter what, even if there are no job finishes to call it
+    {
+//         collection()->collectionUpdated();
+        m_collection->scanManager()->setBlockScan( false );
+        slotRemoveOperationFinished();
+    }
 }
 
 void
@@ -235,6 +381,7 @@ SqlCollectionLocation::insertTracks( const QMap<Meta::TrackPtr, QString> &trackM
         urls.append( trackMap[ track ] );
     }
     ScanResultProcessor processor( m_collection );
+    processor.setSqlStorage( m_collection->sqlStorage() );
     processor.setScanType( ScanResultProcessor::IncrementalScan );
     QMap<QString, uint> mtime = updatedMtime( urls );
     foreach( const QString &dir, mtime.keys() )
@@ -291,22 +438,22 @@ SqlCollectionLocation::updatedMtime( const QStringList &urls )
 void
 SqlCollectionLocation::insertStatistics( const QMap<Meta::TrackPtr, QString> &trackMap )
 {
-    MountPointManager *mpm = MountPointManager::instance();
+    SqlMountPointManager *mpm = m_collection->mountPointManager();
     foreach( const Meta::TrackPtr &track, trackMap.keys() )
     {
         QString url = trackMap[ track ];
         int deviceid = mpm->getIdForUrl( url );
         QString rpath = mpm->getRelativePath( deviceid, url );
         QString sql = QString( "SELECT COUNT(*) FROM statistics LEFT JOIN urls ON statistics.url = urls.id "
-                               "WHERE urls.deviceid = %1 AND urls.rpath = '%2';" ).arg( QString::number( deviceid ), m_collection->escape( rpath ) );
-        QStringList count = m_collection->query( sql );
+                               "WHERE urls.deviceid = %1 AND urls.rpath = '%2';" ).arg( QString::number( deviceid ), m_collection->sqlStorage()->escape( rpath ) );
+        QStringList count = m_collection->sqlStorage()->query( sql );
         if( count.isEmpty() || count.first().toInt() != 0 )    //crash if the sql is bad
         {
             continue;   //a statistics row already exists for that url, and we cannot merge the statistics
         }
         //the row will exist because this method is called after insertTracks
-        QString select = QString( "SELECT id FROM urls WHERE deviceid = %1 AND rpath = '%2';" ).arg( QString::number( deviceid ), m_collection->escape( rpath ) );
-        QStringList result = m_collection->query( select );
+        QString select = QString( "SELECT id FROM urls WHERE deviceid = %1 AND rpath = '%2';" ).arg( QString::number( deviceid ), m_collection->sqlStorage()->escape( rpath ) );
+        QStringList result = m_collection->sqlStorage()->query( select );
         if( result.isEmpty() )
         {
             warning() << "SQL Query returned no results:" << select;
@@ -318,14 +465,14 @@ SqlCollectionLocation::insertStatistics( const QMap<Meta::TrackPtr, QString> &tr
         QString data = "%1,%2,%3,%4,%5,%6";
         data = data.arg( id, QString::number( track->rating() ), QString::number( track->score() ),
                     QString::number( track->playCount() ), QString::number( track->lastPlayed() ), QString::number( track->firstPlayed() ) );
-        m_collection->insert( insert.arg( data ), "statistics" );
+        m_collection->sqlStorage()->insert( insert.arg( data ), "statistics" );
     }
 }
 
 bool SqlCollectionLocation::startNextJob()
 {
     DEBUG_BLOCK
-    if ( !m_sources.isEmpty() )
+    while ( !m_sources.isEmpty() )
     {
         Meta::TrackPtr track = m_sources.keys().first();
         KUrl src = m_sources.take( track );
@@ -349,16 +496,17 @@ bool SqlCollectionLocation::startNextJob()
             {
                 warning() << "Could not create directory " << dir;
                 source()->transferError(track, i18n( "Could not create directory: %1", dir.path() ) );
-                return false;
+                continue; // Attempt to copy/move the next item in m_sources
             }
         }
         if( src == dest) {
         //no changes, so leave the database alone, and don't erase anything
-            return false;
+            continue; // Attempt to copy/move the next item in m_sources
         }
-    //we should only move it directly if we're moving within the same collection
+        //we should only move it directly if we're moving within the same collection
         else if( isGoingToRemoveSources() && source()->collection() == collection() )
         {
+            debug() << "moving!";
             job = KIO::file_move( src, dest, -1, flags );
         }
         else
@@ -373,10 +521,41 @@ bool SqlCollectionLocation::startNextJob()
             if( track->artist() )
                 name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
 
-            The::statusBar()->newProgressOperation( job, i18n( "Transferring: %1", name ) );
+//            The::statusBar()->newProgressOperation( job, i18n( "Transferring: %1", name ) );
             m_jobs.insert( job, track );
             return true;
         }
+        break;
+    }
+    return false;
+}
+
+bool SqlCollectionLocation::startNextRemoveJob()
+{
+    DEBUG_BLOCK
+    while ( !m_removetracks.isEmpty() )
+    {
+        Meta::TrackPtr track = m_removetracks.takeFirst();
+        KUrl src = track->playableUrl();
+
+        KIO::DeleteJob *job = 0;
+
+        src.cleanPath();
+        debug() << "deleting  " << src;
+        KIO::JobFlags flags = KIO::HideProgressInfo;
+        job = KIO::del(src, flags);
+        if( job )   //just to be safe
+        {
+            connect( job, SIGNAL( result(KJob*) ), SLOT( slotRemoveJobFinished(KJob*) ) );
+            QString name = track->prettyName();
+            if( track->artist() )
+                name = QString( "%1 - %2" ).arg( track->artist()->name(), track->prettyName() );
+
+            The::statusBar()->newProgressOperation( job, i18n( "Removing: %1", name ) );
+            m_removejobs.insert( job, track );
+            return true;
+        }
+        break;
     }
     return false;
 }
