@@ -54,10 +54,13 @@ namespace Solid {
 
 class UmsCollection;
 
+class KDialog;
 class KDirLister;
 class KFileItem;
 class KUrl;
 
+class UmsPodcastProvider;
+
 class QAction;
 class QString;
 class QMutex;
@@ -100,6 +103,8 @@ class UmsHandler : public Meta::MediaDeviceHandler
         QString mountPoint() const { return m_mountPoint; }
         void setMountPoint( const QString &mp ) { m_mountPoint = mp; }
 
+        QStringList mimetypes() { return m_mimetypes; }
+
     public slots:
 
     protected:
@@ -147,6 +152,10 @@ class UmsHandler : public Meta::MediaDeviceHandler
         virtual void libSetCoverArt( Itdb_Track *umstrack, const QPixmap &image );
         virtual void setCoverArt( Itdb_Track *umstrack, const QString &path );
 #endif
+    private slots:
+        void slotConfigure();
+        void slotConfigChanged();
+
     private:
         enum FileType
         {
@@ -255,6 +264,13 @@ class UmsHandler : public Meta::MediaDeviceHandler
         KTempDir *m_tempdir;
         QSet<QString> m_coverArt;
 
+        //direct implementation of a podcast provider NOT using the MD::Capabilities
+        UmsPodcastProvider *m_podcastProvider;
+        QAction *m_configureAction;
+        QString m_podcastPath;
+
+        KDialog *m_umsSettingsDialog;
+
     private slots:
         void slotCreateEntry( const QString &path );
         void slotCheckDirty();
