@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2004-2007 Mark Kretschmann <kretschmann@kde.org>                       *
+ * Copyright (c) 2004-2010 Mark Kretschmann <kretschmann@kde.org>                       *
  * Copyright (c) 2005-2007 Seb Ruiz <ruiz@kde.org>                                      *
  * Copyright (c) 2006 Alexandre Pereira de Oliveira <aleprj@gmail.com>                  *
  * Copyright (c) 2006 Martin Ellis <martin.ellis@kdemail.net>                           *
@@ -65,17 +65,11 @@
 
 #if KDE_IS_VERSION(4, 3, 80)
 #define HAVE_KNEWSTUFF3
-#endif
-
-#ifdef HAVE_KNEWSTUFF3
-#include <knewstuff2/engine.h>
-#include <knewstuff2/core/entry.h>
 #include <KNS3/DownloadDialog>
-#else
-#include <knewstuff2/engine.h>
-#include <knewstuff2/core/entry.h>
 #endif
 
+#include <knewstuff2/engine.h>
+#include <knewstuff2/core/entry.h>
 
 #include <QFileInfo>
 #include <QTimer>
@@ -141,6 +135,9 @@ ScriptManager::ScriptManager( QWidget* parent )
     // Delay this call via eventloop, because it's a bit slow and would block
     QTimer::singleShot( 0, this, SLOT( updateAllScripts() ) );
 
+
+    //FIXME This is only here for testing purposes. Remove later.
+    //QTimer::singleShot( 0, this, SLOT( showScriptStalledDialog() ) );
 }
 
 ScriptManager::~ScriptManager()
@@ -580,6 +577,23 @@ ScriptManager::scriptFinished( QString name ) //SLOT
     delete m_scripts[name].engine;
 }
 
+
+//MOCKUP method, see API docs
+void
+ScriptManager::showScriptStalledDialog()  // SLOT
+{
+    const QString script = "FIXME";
+
+    const int reply = KMessageBox::questionYesNo( 0, i18n( "The script '%1' appears to have stalled.\n\n"
+                                                           "Would you like to stop it?", script ),
+                                                           i18n( "Script Manager - Amarok" ),
+                                                           KStandardGuiItem::yes(),
+                                                           KStandardGuiItem::no(),
+                                                           QString(),
+                                                           KMessageBox::Dangerous );
+}
+
+
 ////////////////////////////////////////////////////////////////////////////////
 // private
 ////////////////////////////////////////////////////////////////////////////////
