@@ -40,14 +40,14 @@ public:
      * @param seconds the position in seconds at which the timecide should be stored.
      * @return  true if the write was successful, false if not.
      */
-    virtual bool writeTimecode ( int seconds ) = 0;
+    virtual bool writeTimecode ( qint64 miliseconds ) = 0;
 
     /**
      * Stores an auto timecode for the track and deletes any previously added auto timecodes
      * @param seconds the position in seconds at which the timecide should be stored.
      * @return  true if the write was successful, false if not.
      */
-    virtual bool writeAutoTimecode ( int seconds ) = 0;
+    virtual bool writeAutoTimecode ( qint64 miliseconds ) = 0;
 
     /**
     * Get the capabilityInterfaceType of this capability
@@ -60,8 +60,8 @@ public:
 
 protected:
 
-    bool writeTimecode( int seconds, Meta::TrackPtr track );
-    bool writeAutoTimecode( int seconds, Meta::TrackPtr track );
+    bool writeTimecode( qint64 miliseconds, Meta::TrackPtr track );
+    bool writeAutoTimecode( qint64 miliseconds, Meta::TrackPtr track );
 };
 
 #endif // TIMECODEWRITECAPABILITY_H
