@@ -32,11 +32,13 @@ namespace Playlist
         public:
             RepeatTrackNavigator();
 
+            quint64 likelyNextTrack() { return m_trackid; }
+            quint64 likelyLastTrack() { return m_trackid; }
             quint64 requestNextTrack() { return m_trackid; }
             quint64 requestLastTrack() { return m_trackid; }
 
             virtual void reset() {};
-            
+
         private slots:
             void recvActiveTrackChanged( const quint64 id ) { m_trackid = id; }
 
