@@ -70,14 +70,16 @@ namespace AmarokScript
         signals:
             void trackFinished(); // when playback stops altogether
             void trackChanged();
+            void newMetaData( const QHash<qint64, QString> &newMetaData, bool trackChanged );
             void trackSeeked( int ); //return relative time in million second
             void volumeChanged( int );
             void trackPlayPause( int );  //Playing: 0, Paused: 1
 
         private:
             void engineVolumeChanged( int value );
-            void engineTrackPositionChanged( long position, bool userSeek );
+            void engineTrackPositionChanged( qint64 position, bool userSeek );
             void engineTrackChanged( Meta::TrackPtr track );
+            void engineNewMetaData( const QHash<qint64, QString> &newMetaData, bool trackChanged );
             void engineStateChanged( Phonon::State currentState, Phonon::State oldState );
 
             bool randomMode() const;
