@@ -20,7 +20,7 @@
 
 #include "AvatarDownloader.h"
 #include "EngineController.h"
-#include "biases/SimilarArtistsBias.h"
+#include "biases/LastFmBias.h"
 #include "biases/WeeklyTopBias.h"
 #include "browsers/CollectionTreeItem.h"
 #include "browsers/CollectionTreeItemModelBase.h"
@@ -38,7 +38,7 @@
 #include "collection/CollectionManager.h"
 #include "meta/capabilities/LastFmCapability.h"
 #include "meta/LastFmMeta.h"
-#include "playlist/PlaylistController.h"
+#include "playlist/PlaylistModelStack.h"
 #include "widgets/SearchWidget.h"
 #include "CustomBias.h"
 
@@ -183,7 +183,11 @@ LastFmService::LastFmService( LastFmServiceFactory* parent, const QString &name,
       m_profile( 0 ),
       m_userinfo( 0 ),
       m_userName( username ),
-      m_sessionKey( sessionKey )
+      m_sessionKey( sessionKey ),
+      m_userNameArray( 0 ),
+      m_sessionKeyArray( 0 ),
+      m_lastFmBiasFactory( 0 ),
+      m_weeklyTopBiasFactory( 0 )
 {
     DEBUG_BLOCK
 
@@ -205,7 +209,12 @@ LastFmService::~LastFmService()
 {
     DEBUG_BLOCK
 
-    if( m_collection )
+    delete m_lastFmBiasFactory;
+    delete m_weeklyTopBiasFactory;
+    delete[] m_userNameArray;
+    delete[] m_sessionKeyArray;
+
+    if( m_collection && CollectionManager::instance() )
     {
         CollectionManager::instance()->removeUnmanagedCollection( m_collection );
         delete m_collection;
@@ -221,18 +230,19 @@ LastFmService::init()
     const QString password = config.password();
     const QString sessionKey = config.sessionKey();
     // set the global static Lastfm::Ws stuff
-    lastfm::ws::ApiKey = "402d3ca8e9bc9d3cf9b85e1202944ca5";
+    lastfm::ws::ApiKey = Amarok::lastfmApiKey();
     lastfm::ws::SharedSecret = "fe0dcde9fcd14c2d1d50665b646335e9";
     // testing w/ official keys
     //Ws::SharedSecret = "73582dfc9e556d307aead069af110ab8";
     //Ws::ApiKey = "c8c7b163b11f92ef2d33ba6cd3c2c3c3";
-    lastfm::ws::Username = qstrdup( m_userName.toLatin1().data() );
+    m_userNameArray = qstrdup( m_userName.toLatin1().data() );
+    lastfm::ws::Username = m_userNameArray;
 
 
     // set up proxy
     QNetworkAccessManager* qnam = new KNetworkAccessManager( this );
     lastfm::setNetworkAccessManager( qnam );
-    
+
     debug() << "username:" << QString( QUrl::toPercentEncoding( lastfm::ws::Username ) );
 
     QString authToken =  md5( ( m_userName + md5( password.toUtf8() ) ).toUtf8() );
@@ -252,7 +262,8 @@ LastFmService::init()
     } else
     {
         debug() << "using saved sessionkey from last.fm";
-        lastfm::ws::SessionKey = qstrdup( sessionKey.toLatin1().data() );
+        m_sessionKeyArray = qstrdup( sessionKey.toLatin1().data() );
+        lastfm::ws::SessionKey = m_sessionKeyArray;
         m_sessionKey = sessionKey;
 
         if( m_scrobble )
@@ -269,13 +280,12 @@ LastFmService::init()
     m_searchWidget->setVisible( false );
 
     // enable custom bias
-    Dynamic::SimilarArtistsBiasFactory* similarF = new Dynamic::SimilarArtistsBiasFactory();
-    Dynamic::CustomBias::registerNewBiasFactory( similarF );
+    m_lastFmBiasFactory = new Dynamic::LastFmBiasFactory();
+    Dynamic::CustomBias::registerNewBiasFactory( m_lastFmBiasFactory );
+
+    m_weeklyTopBiasFactory = new Dynamic::WeeklyTopBiasFactory();
+    Dynamic::CustomBias::registerNewBiasFactory( m_weeklyTopBiasFactory );
 
-    // disabled until I figure out how to get what I want from last.fm
-    //Dynamic::WeeklyTopBiasFactory* weeklyF = new Dynamic::WeeklyTopBiasFactory();
-    //Dynamic::CustomBias::registerNewBiasFactory( weeklyF );
-    
     m_collection = new LastFmServiceCollection( m_userName );
     CollectionManager::instance()->addUnmanagedCollection( m_collection, CollectionManager::CollectionDisabled );
 
@@ -325,7 +335,8 @@ LastFmService::onAuthenticated()
             }
             m_sessionKey = lfm[ "session" ][ "key" ].text();
 
-            lastfm::ws::SessionKey = qstrdup( m_sessionKey.toLatin1().data() );
+            m_sessionKeyArray = qstrdup( m_sessionKey.toLatin1().data() );
+            lastfm::ws::SessionKey = m_sessionKeyArray;
             config.setSessionKey( m_sessionKey );
             config.save();
 
@@ -556,21 +567,22 @@ LastFmService::love()
 
     Meta::TrackPtr track = The::engineController()->currentTrack();
     LastFm::Track* lastfmTrack = dynamic_cast< LastFm::Track* >( track.data() );
-    if( track )
-        The::statusBar()->shortMessage( i18nc( "As in, lastfm", "Loved Track: %1", track->prettyName() ) );
 
     if( lastfmTrack )
+    {
         lastfmTrack->love();
+        The::statusBar()->shortMessage( i18nc( "As in, lastfm", "Loved Track: %1", track->prettyName() ) );
+    }
     else
+    {
         m_scrobbler->loveTrack( track );
+    }
 
 }
 
 void LastFmService::love( Meta::TrackPtr track )
 {
     DEBUG_BLOCK
-    if( track )
-        The::statusBar()->shortMessage( i18nc( "As in, lastfm", "Loved Track: %1", track->prettyName() ) );
     m_scrobbler->loveTrack( track );
 }
 
