@@ -35,8 +35,8 @@ public:
     ScrobblerAdapter( QObject *parent, const QString &clientId );
     virtual ~ScrobblerAdapter();
 
-    virtual void enginePlaybackEnded( int finalPosition, int trackLength, PlaybackEndedReason reason );
-    virtual void engineTrackPositionChanged( long position , bool userSeek );
+    virtual void enginePlaybackEnded( qint64 finalPosition, qint64 trackLength, PlaybackEndedReason reason );
+    virtual void engineTrackPositionChanged( qint64 position , bool userSeek );
     virtual void engineNewTrackPlaying();
     virtual void engineNewMetaData( const QHash<qint64, QString> &newMetaData, bool trackChanged ); // for stream scrobbling
 
@@ -46,6 +46,7 @@ public:
 
 public slots:
     void loveTrack( Meta::TrackPtr );
+    void banTrack();
 
 private:
     void resetVariables();
@@ -53,11 +54,11 @@ private:
 
     lastfm::Audioscrobbler *m_scrobbler;
     lastfm::MutableTrack m_current;
-    long m_lastPosition;
-    uint m_totalPlayed;
+    qint64 m_lastPosition;
+    qint64 m_totalPlayed;
     QString m_clientId;
 
-    long int m_lastSaved;
+    qint64 m_lastSaved;
 };
 
 #endif // LASTFMSCROBBLERADAPTER_H
