@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2009 Nikolaj Hald Nielsen <nhnFreespirit@gmail.com>                    *
+ * Copyright (c) 2009 Nikolaj Hald Nielsen <nhn@kde.org>                                *
  * Copyright (c) 2009 Mark Kretschmann <kretschmann@kde.org>                            *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
@@ -23,6 +23,7 @@
 #include <QToolButton>
 
 class QAction;
+class QEvent;
 class QLabel;
 class QMenu;
 class QMouseEvent;
@@ -39,8 +40,9 @@ public:
     VolumePopupButton( QWidget * parent );
     
 protected:
-    void mouseReleaseEvent( QMouseEvent * event );
-    void wheelEvent( QWheelEvent * event );
+    virtual void mouseReleaseEvent( QMouseEvent * event );
+    virtual void wheelEvent( QWheelEvent * event );
+    virtual bool eventFilter( QObject *object, QEvent *event );
 
 private:
     void engineVolumeChanged( int newVolume );
