@@ -34,14 +34,10 @@ class ProgressWidget : public QWidget, public EngineObserver
     Q_OBJECT
     public:
         ProgressWidget( QWidget* );
-        ~ProgressWidget();
 
         virtual QSize sizeHint() const;
-
-        void addBookmark( const QString &name, int milliSeconds , bool instantDisplayPopUp = false );
-        void redrawBookmarks(const QString *BookmarkName = 0);
-
-       Amarok::TimeSlider* slider() const { return m_slider; }
+        void addBookmark( const QString &name, int milliSeconds , bool instantDisplayPopUp );
+        Amarok::TimeSlider* slider() const { return m_slider; }
 
     public slots:
         void drawTimeDisplay( int position );
@@ -52,7 +48,11 @@ class ProgressWidget : public QWidget, public EngineObserver
         virtual void engineTrackLengthChanged( qint64 milliseconds );
         virtual void engineNewTrackPlaying();
         virtual void enginePlaybackEnded( qint64 finalPosition, qint64 trackLength, PlaybackEndedReason reason );
-
+        
+    private slots:
+        void addBookmark( const QString &name, int milliSeconds );
+        void redrawBookmarks(const QString *BookmarkName = 0);
+        
     private:
         TimeLabel *m_timeLabelLeft;
         TimeLabel *m_timeLabelRight;
