@@ -20,14 +20,16 @@
 #ifndef TESTCASECONVERTER_H
 #define TESTCASECONVERTER_H
 
+#include "TestBase.h"
+
 #include <QtCore/QObject>
 
-class TestCaseConverter : public QObject
+class TestCaseConverter : public TestBase
 {
 Q_OBJECT
 
 public:
-    TestCaseConverter( QStringList testArgumentList, bool stdout );
+    TestCaseConverter( const QStringList args, const QString &logPath );
 
 private slots:
     void testToCapitalizedCase();
